@@ -1,17 +1,18 @@
 package oviewer
 
 import (
+	"context"
+
 	"github.com/gdamore/tcell/v2"
 )
 
-// setJumpTargetMode sets the inputMode to JumpTarget.
-func (root *Root) setJumpTargetMode() {
+// inputJumpTarget sets the inputMode to JumpTarget.
+func (root *Root) inputJumpTarget(context.Context) {
 	input := root.input
 	input.reset()
 
-	input.JumpTargetCandidate.toLast(root.Doc.JumpTarget)
-
-	input.Event = newJumpTargetEvent(input.JumpTargetCandidate)
+	input.Candidate[JumpTarget].toLast(root.Doc.JumpTarget)
+	input.Event = newJumpTargetEvent(input.Candidate[JumpTarget])
 }
 
 // jumpTargetCandidate returns the candidate to set to default.
@@ -42,7 +43,7 @@ func (*eventJumpTarget) Mode() InputMode {
 
 // Prompt returns the prompt string in the input field.
 func (*eventJumpTarget) Prompt() string {
-	return "Jump Target line:"
+	return "Jump target line:"
 }
 
 // Confirm returns the event when the input is confirmed.
