@@ -12,6 +12,7 @@ const bottomMargin = 2
 
 // Go to the top line.
 func (root *Root) moveTop(context.Context) {
+	root.setPauseFollow()
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
@@ -24,10 +25,13 @@ func (root *Root) moveBottom(context.Context) {
 	defer root.releaseEventBuffer()
 
 	root.Doc.moveBottom()
+	// Follow mode is unconditionally resumed when moving to the bottom.
+	root.Doc.pauseFollow = false
 }
 
 // Move up one screen.
 func (root *Root) movePgUp(context.Context) {
+	root.setPauseFollow()
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
@@ -39,11 +43,14 @@ func (root *Root) movePgDn(context.Context) {
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
-	root.Doc.movePgDn()
+	if root.Doc.movePgDn() {
+		root.Doc.pauseFollow = false
+	}
 }
 
 // Moves up half a screen.
 func (root *Root) moveHfUp(context.Context) {
+	root.setPauseFollow()
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
@@ -55,11 +62,14 @@ func (root *Root) moveHfDn(context.Context) {
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
-	root.Doc.moveHfDn()
+	if root.Doc.moveHfDn() {
+		root.Doc.pauseFollow = false
+	}
 }
 
 // Move up one line.
 func (root *Root) moveUpOne(context.Context) {
+	root.setPauseFollow()
 	root.moveUp(1)
 }
 
@@ -70,6 +80,7 @@ func (root *Root) moveDownOne(context.Context) {
 
 // Move up by n amount.
 func (root *Root) moveUp(n int) {
+	root.setPauseFollow()
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
@@ -81,7 +92,9 @@ func (root *Root) moveDown(n int) {
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
-	root.Doc.moveYDown(n)
+	if root.Doc.moveYDown(n) {
+		root.Doc.pauseFollow = false
+	}
 }
 
 // nextSection moves down to the next section's delimiter.
@@ -90,6 +103,7 @@ func (root *Root) nextSection(ctx context.Context) {
 	defer root.releaseEventBuffer()
 
 	if err := root.Doc.moveNextSection(ctx); err != nil {
+		root.Doc.pauseFollow = false
 		// Move by page if there is no section.
 		root.Doc.movePgDn()
 		// Last section or no section.
@@ -97,10 +111,19 @@ func (root *Root) nextSection(ctx context.Context) {
 			root.setMessage("No more next sections")
 		}
 	}
+
+	// Cancel quit small mode.
+	// Normally determined after file load, but prevents quitting when input is slow (e.g., from stdin).
+	// No separate key binding due to its specific use case.
+	if root.Config.QuitSmall {
+		root.Config.QuitSmall = false
+		root.setMessage("Quit small mode canceled")
+	}
 }
 
 // prevSection moves up to the delimiter of the previous section.
 func (root *Root) prevSection(ctx context.Context) {
+	root.setPauseFollow()
 	root.resetSelect()
 	defer root.releaseEventBuffer()
 
@@ -173,7 +196,7 @@ func (root *Root) moveHfLeft(context.Context) {
 	defer root.releaseEventBuffer()
 
 	root.Doc.moveHfLeft()
-	root.Doc.x = max(root.Doc.x, root.minStartX)
+	root.Doc.scrollX = max(root.Doc.scrollX, root.minStartX)
 }
 
 // Move to the right by half a screen.
@@ -186,7 +209,7 @@ func (root *Root) moveHfRight(context.Context) {
 
 // moveBeginLeft moves to the beginning of the line.
 func (root *Root) moveBeginLeft(context.Context) {
-	root.Doc.moveBeginLeft()
+	root.Doc.moveBeginLeft(root.scr)
 }
 
 // moveEndRight moves to the end of the line.
@@ -198,7 +221,7 @@ func (root *Root) moveEndRight(context.Context) {
 // moveNormalLeft moves the screen left.
 func (root *Root) moveNormalLeft(n int) {
 	root.Doc.moveNormalLeft(n)
-	root.Doc.x = max(root.Doc.x, root.minStartX)
+	root.Doc.scrollX = max(root.Doc.scrollX, root.minStartX)
 }
 
 // moveNormalRight moves the screen right.
@@ -209,6 +232,7 @@ func (root *Root) moveNormalRight(n int) {
 // moveColumnLeft moves the cursor to the left by n amount.
 func (root *Root) moveColumnLeft(n int) {
 	if err := root.Doc.moveColumnLeft(n, root.scr, !root.Config.DisableColumnCycle); err != nil {
+		root.setMessagef("Cannot move column: %s", err)
 		root.debugMessage(err.Error())
 	}
 }
@@ -216,6 +240,7 @@ func (root *Root) moveColumnLeft(n int) {
 // moveColumnRight moves the cursor to the right by n amount.
 func (root *Root) moveColumnRight(n int) {
 	if err := root.Doc.moveColumnRight(n, root.scr, !root.Config.DisableColumnCycle); err != nil {
+		root.setMessagef("Cannot move column: %s", err)
 		root.debugMessage(err.Error())
 	}
 }
