@@ -11,7 +11,9 @@ import (
 	"path/filepath"
 	"regexp"
 	"sort"
+	"strings"
 	"sync"
+	"sync/atomic"
 	"syscall"
 
 	"codeberg.org/tslocum/cbind"
@@ -19,6 +21,7 @@ import (
 	"github.com/gdamore/tcell/v2"
 	"github.com/noborus/tcellansi"
 	"github.com/spf13/viper"
+	"golang.org/x/term"
 )
 
 // Root is the root structure of the oviewer.
@@ -85,6 +88,8 @@ type Root struct {
 
 	// mu controls the RWMutex.
 	mu sync.RWMutex
+	// isClosed indicates whether it is closed.
+	isClosed atomic.Bool
 
 	// FollowAll is a follow mode for all documents.
 	FollowAll bool
@@ -92,6 +97,20 @@ type Root struct {
 	skipDraw bool
 	// clickState is the state of mouse click.
 	clickState ClickState
+
+	// sidebarVisible is whether the sidebar is visible.
+	sidebarVisible bool
+	// sidebarMode is the mode of the sidebar.
+	sidebarMode         SidebarMode
+	previousSidebarMode SidebarMode
+	// SidebarItems is the items to display in the sidebar.
+	SidebarItems []SidebarItem
+	// SidebarHelpItems is the help items to display in the sidebar.
+	SidebarHelpItems []SidebarItem
+	// sidebarWidth is the width of the sidebar.
+	sidebarWidth int
+	// sidebarScrolls holds scroll positions for each sidebarMode.
+	sidebarScrolls map[SidebarMode]sidebarScroll
 }
 
 // MouseSelectState represents the state of mouse selection.
@@ -114,10 +133,6 @@ type SCR struct {
 	vWidth int
 	// vHeight represents the screen height.
 	vHeight int
-	// startX is the start position of x.
-	startX int
-	// startY is the start position of y.
-	startY int
 	// statusLineHeight is the height of the status line.
 	statusLineHeight int
 
@@ -461,6 +476,7 @@ func NewOviewer(docs ...*Document) (*Root, error) {
 		keyConfig:      cbind.NewConfiguration(),
 		inputKeyConfig: cbind.NewConfiguration(),
 		input:          NewInput(),
+		sidebarScrolls: make(map[SidebarMode]sidebarScroll),
 	}
 	root.DocList = append(root.DocList, docs...)
 	root.Doc = root.DocList[0]
@@ -628,12 +644,10 @@ func openFiles(fileNames []string) (*Root, error) {
 		docList = append(docList, m)
 	}
 
-	if len(openErrs) > 1 {
-		openErrs = append([]error{ErrMissingFile}, openErrs...)
-	}
-	errs := errors.Join(openErrs...)
 	// If none of the documents are present, the program exits with an error.
 	if len(docList) == 0 {
+		openErrs = append([]error{ErrMissingFile}, openErrs...)
+		errs := errors.Join(openErrs...)
 		return nil, errs
 	}
 
@@ -642,9 +656,9 @@ func openFiles(fileNames []string) (*Root, error) {
 		return nil, err
 	}
 	// Errors that could not be OpenDocument are output to the log.
-	if errs, ok := errs.(interface{ Unwrap() []error }); ok {
-		for _, err := range errs.Unwrap() {
-			log.Println(err)
+	if len(openErrs) > 0 {
+		for _, e := range openErrs {
+			log.Println(e)
 		}
 	}
 
@@ -662,14 +676,24 @@ func (root *Root) SetConfig(config Config) {
 
 	// view mode.
 	if config.ViewMode != "" {
-		viewMode, overwrite := config.Mode[config.ViewMode]
-		if overwrite {
+		viewMode, ok := config.Mode[config.ViewMode]
+		if ok {
 			root.settings = updateRunTimeSettings(root.settings, viewMode)
 		} else {
 			root.setMessageLogf("view mode not found: %s", config.ViewMode)
 		}
 	}
 
+	if config.SidebarMode != "" {
+		for s := SidebarModeHelp; s < SidebarModeEnd; s++ {
+			if strings.EqualFold(config.SidebarMode, s.String()) {
+				root.sidebarMode = s
+				root.sidebarVisible = true
+				break
+			}
+		}
+	}
+
 	// Set the follow mode for all documents.
 	root.FollowAll = root.settings.FollowAll
 	// Set the minimum start position of x.
@@ -787,7 +811,6 @@ func (root *Root) Run() error {
 	}
 	defer watcher.Close()
 	root.SetWatcher(watcher)
-
 	if err := root.prepareRun(ctx); err != nil {
 		return err
 	}
@@ -838,12 +861,18 @@ func (root *Root) prepareRun(ctx context.Context) error {
 	}
 
 	if root.Config.ShrinkChar != "" {
-		Shrink = []rune(root.Config.ShrinkChar)[0]
+		Shrink = root.Config.ShrinkChar
 		SetShrinkContent(Shrink)
 	}
 
 	root.setCaption()
 
+	// Validate sidebar width.
+	if _, err = calcSideWidth(root.Config.SidebarWidth, 100); err != nil {
+		log.Printf("invalid sidebar width: %s, set to default %s\n", root.Config.SidebarWidth, defaultSidebarWidth)
+		root.Config.SidebarWidth = defaultSidebarWidth
+	}
+
 	root.setViewModeConfig()
 	root.prepareAllDocuments()
 	// follow mode or follow all disables quit if the output fits on one screen.
@@ -896,9 +925,6 @@ func (root *Root) prepareAllDocuments() {
 		if doc.ColumnWidth {
 			doc.ColumnMode = true
 		}
-		if doc.Converter != "" {
-			doc.conv = doc.converterType(doc.Converter)
-		}
 		w := ""
 		if doc.WatchInterval > 0 {
 			doc.watchMode()
@@ -913,6 +939,7 @@ func (root *Root) prepareAllDocuments() {
 
 // Close closes the oviewer.
 func (root *Root) Close() {
+	root.isClosed.Store(true)
 	root.Screen.Fini()
 }
 
@@ -1294,12 +1321,12 @@ func (root *Root) outputOnExit(output io.Writer) {
 func (root *Root) writeCurrentScreen(output io.Writer) {
 	strs := root.OnExit
 	if strs == nil {
-		height, err := root.dummyScreen()
+		start, end, err := root.dummyScreen()
 		if err != nil {
 			log.Println(err)
 			return
 		}
-		strs = tcellansi.ScreenContentToStrings(root.Screen, 0, root.Doc.width, 0, height)
+		strs = tcellansi.ScreenContentToStrings(root.Screen, 0, root.Doc.bodyWidth, start, end)
 		strs = tcellansi.TrimRightSpaces(strs)
 	}
 	for _, str := range strs {
@@ -1311,10 +1338,20 @@ func (root *Root) writeCurrentScreen(output io.Writer) {
 }
 
 // dummyScreen creates a dummy screen.
-func (root *Root) dummyScreen() (int, error) {
+func (root *Root) dummyScreen() (int, int, error) {
+	col := 80
+	row := 25
+	fd := int(os.Stdout.Fd())
+	w, h, err := term.GetSize(fd)
+	if err == nil && w > 0 && h > 0 {
+		col = w
+		row = h
+	}
+	root.scr.vWidth = col
+	root.scr.vHeight = row
 	root.Screen = tcell.NewSimulationScreen("")
 	if err := root.Screen.Init(); err != nil {
-		return 0, err
+		return 0, 0, err
 	}
 	height := root.scr.vHeight
 	if root.Config.BeforeWriteOriginal != 0 || root.Config.AfterWriteOriginal != 0 {
@@ -1333,26 +1370,40 @@ func (root *Root) dummyScreen() (int, error) {
 	root.prepareScreen()
 	root.prepareDraw(ctx)
 	root.draw(ctx)
-	height = realHeight(root.scr)
-	return height, nil
+	start, end := contentRange(root.scr)
+	return start, end, nil
 }
 
-func realHeight(scr SCR) int {
-	height := 0
-	for ; height < scr.vHeight-scr.statusLineHeight; height++ {
-		if !scr.lines[scr.numbers[height].number].valid {
+// contentRange returns the range of valid content on the screen.
+func contentRange(scr SCR) (int, int) {
+	start := 0
+	end := 0
+	valid := false
+	for y := 0; y < scr.vHeight-scr.statusLineHeight; y++ {
+		if y >= len(scr.numbers) {
 			break
 		}
+		if !scr.lines[scr.numbers[y].number].valid {
+			continue
+		}
+		if !valid {
+			start = y
+			valid = true
+		}
+		end = y + 1
 	}
-	return height
+	return start, end
 }
 
 // ScreenContent returns the screen content.
 func (root *Root) ScreenContent() []string {
 	root.Screen.Sync()
 	m := root.Doc
-	height := realHeight(root.scr)
-	strs := tcellansi.ScreenContentToStrings(root.Screen, 0, m.width, 0, height)
+	start, end := contentRange(root.scr)
+	end += root.Config.AfterWriteOriginal
+	start -= root.Config.BeforeWriteOriginal
+	end = max(end, start)
+	strs := tcellansi.ScreenContentToStrings(root.Screen, 0, m.bodyWidth, start, end)
 	strs = tcellansi.TrimRightSpaces(strs)
 	return strs
 }
