@@ -26,9 +26,9 @@ func sectionHeader2Helper(t *testing.T) *Root {
 func sectionHeaderTextHelper(t *testing.T, fileName string) *Root {
 	t.Helper()
 	root := rootFileReadHelper(t, filepath.Join(testdata, fileName))
-	m := root.Doc
-	m.width = 80
+	root.scr.vWidth = 80
 	root.scr.vHeight = 24
+	m := root.Doc
 	m.topLX = 0
 	root.scr.lines = make(map[int]LineC)
 	return root
@@ -643,7 +643,7 @@ func TestRoot_searchHighlight(t *testing.T) {
 
 			lineC := root.Doc.getLineC(tt.args.lineNum)
 			root.searcher = tt.fields.searcher
-			root.StyleSearchHighlight = OVStyle{Reverse: true}
+			root.Config.StyleSearchHighlight = OVStyle{Reverse: true}
 			root.searchHighlight(lineC)
 			if lineC.str != tt.want.str {
 				t.Errorf("\nline: %v\nwant: %v\n", lineC.str, tt.want.str)
@@ -742,7 +742,7 @@ func TestRoot_columnDelimiterHighlight(t *testing.T) {
 			m.ColumnDelimiter = tt.fields.columnDelimiter
 			m.ColumnDelimiterReg = condRegexpCompile(m.ColumnDelimiter)
 			m.columnCursor = tt.fields.columnCursor
-			root.StyleColumnHighlight = OVStyle{Bold: true}
+			root.Doc.Style.ColumnHighlight = OVStyle{Bold: true}
 			lineC := root.Doc.getLineC(tt.args.lineNum)
 			lineC.columnRanges = root.Doc.columnDelimiterRange(lineC)
 			root.columnHighlight(lineC)
@@ -808,8 +808,8 @@ func TestRoot_columnWidthHighlight(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			root := rootFileReadHelper(t, filepath.Join(testdata, "ps.txt"))
-			root.StyleColumnHighlight = OVStyle{Bold: true}
 			m := root.Doc
+			m.Style.ColumnHighlight = OVStyle{Bold: true}
 			m.ColumnWidth = true
 			m.ColumnMode = true
 			root.prepareScreen()
@@ -974,6 +974,16 @@ func Test_findColumnEnd(t *testing.T) {
 			},
 			want: 31,
 		},
+		{
+			name: "Test findColumnEnd10Over",
+			args: args{
+				str:   "あいうえお かきくけこ さしすせそ",
+				pos:   []int{10, 15},
+				n:     1,
+				start: 10,
+			},
+			want: 21,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
@@ -1145,9 +1155,9 @@ func TestRoot_setAlignConverter(t *testing.T) {
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			root := rootFileReadHelper(t, filepath.Join(testdata, tt.fields.fileName))
-			m := root.Doc
-			m.width = 80
+			root.scr.vWidth = 80
 			root.scr.vHeight = 24
+			m := root.Doc
 			m.topLX = 0
 			m.Header = tt.fields.header
 			m.ColumnMode = true
@@ -1170,6 +1180,63 @@ func TestRoot_setAlignConverter(t *testing.T) {
 	}
 }
 
+func TestRoot_determineStatusLine(t *testing.T) {
+	tcellNewScreen = fakeScreen
+	defer func() {
+		tcellNewScreen = tcell.NewScreen
+	}()
+	root := sectionHeader1Helper(t)
+	root.message = ""
+	root.Doc.StatusLine = false
+	tests := []struct {
+		name       string
+		statusLine bool
+		message    string
+		inputEvent Eventer
+		want       int
+	}{
+		{
+			name:       "StatusLine true",
+			statusLine: true,
+			message:    "",
+			inputEvent: normal(),
+			want:       DefaultStatusLine,
+		},
+		{
+			name:       "StatusLine false, message exists",
+			statusLine: false,
+			message:    "some message",
+			inputEvent: normal(),
+			want:       DefaultStatusLine,
+		},
+		{
+			name:       "StatusLine false, no message, Normal mode",
+			statusLine: false,
+			message:    "",
+			inputEvent: normal(),
+			want:       0,
+		},
+		{
+			name:       "StatusLine false, no message, Insert mode",
+			statusLine: false,
+			message:    "",
+			inputEvent: newGotoEvent(root.input.Candidate[Goline]),
+			want:       DefaultStatusLine,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			root.Doc.StatusLine = tt.statusLine
+			root.message = tt.message
+			root.input.Event = tt.inputEvent
+			got := root.determineStatusLine()
+			if got != tt.want {
+				t.Errorf("determineStatusLine() got %d, want %d", got, tt.want)
+			}
+		})
+	}
+}
+
 func Test_maxWidthsDelm(t *testing.T) {
 	type args struct {
 		maxWidths    []int
