@@ -120,6 +120,9 @@ func (s *store) evictChunksMem(chunkNum int) {
 
 // unloadChunk unloads the chunk from memory.
 func (s *store) unloadChunk(chunkNum int) {
+	s.mu.Lock()
+	defer s.mu.Unlock()
+
 	s.loadedChunks.Remove(chunkNum)
 	s.chunks[chunkNum].lines = nil
 }
@@ -360,6 +363,7 @@ func (s *store) appendFormFeed(chunk *chunk) {
 	}
 }
 
+// export exports the lines including ANSI escape sequences.
 func (s *store) export(w io.Writer, chunk *chunk, start int, end int) error {
 	s.mu.RLock()
 	defer s.mu.RUnlock()
@@ -373,3 +377,19 @@ func (s *store) export(w io.Writer, chunk *chunk, start int, end int) error {
 	}
 	return nil
 }
+
+// exportPlain exports the plain text without ANSI escape sequences.
+func (s *store) exportPlain(w io.Writer, chunk *chunk, start int, end int) error {
+	s.mu.RLock()
+	defer s.mu.RUnlock()
+
+	start = max(0, start)
+	end = min(len(chunk.lines), end)
+	for i := start; i < end; i++ {
+		plain := stripEscapeSequenceBytes(chunk.lines[i])
+		if _, err := w.Write(plain); err != nil {
+			return err
+		}
+	}
+	return nil
+}
